/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "meta/meta.pmh"

using namespace fe;

MetaBrush::MetaBrush(void):
	m_pOperatorContext(NULL)
{
}

MetaBrush::~MetaBrush(void)
{
	delete(m_pOperatorContext);
}

void MetaBrush::initEvent(sp<Scope>& a_rspScope)
{
	m_event.bind(a_rspScope);
	sp<Layout> spLayout=m_event.layout();
	Record record=a_rspScope->createRecord(spLayout);
	m_event.bind(record);
	m_event.reset();
}

void MetaBrush::initDrawInterfaces(MetaPlugin* a_pMetaPlugin)
{
	m_spDrawPerspective=a_pMetaPlugin->drawBrush();
	m_spDrawOrtho=a_pMetaPlugin->drawBrushOverlay();
	sp<DrawCached> spDrawCached=m_spDrawPerspective;
	if(spDrawCached.isValid())
	{
		spDrawCached->setDrawChain(m_spDrawOpenGL);
	}
	spDrawCached=m_spDrawOrtho;
	if(spDrawCached.isValid())
	{
		spDrawCached->setDrawChain(m_spDrawOpenGL);
	}
}

U32 MetaBrush::initFont(void)
{
	//* HACK load font manually
	const String name="-*-clean-medium-r-*-*-8-*-*-*-*-*-*-*";
	Display* pDisplayHandle=XOpenDisplay(NULL);
	XFontStruct* pFontStruct=XLoadQueryFont(pDisplayHandle,name.c_str());
	const U32 lists=pFontStruct->max_char_or_byte2+1;
	const U32 fontBase=glGenLists(lists);

	const Font fontid=pFontStruct->fid;
	const U32 first=pFontStruct->min_char_or_byte2;
	const U32 last=pFontStruct->max_char_or_byte2;

	glXUseXFont(fontid,first,last-first+1,fontBase+first);

	return fontBase;
}

void MetaBrush::setupDrawOpenGL(void)
{
	const U32 fontBase=initFont();
	m_spDrawOpenGL->setFontBase(fontBase);

	sp<DrawMode> spDrawMode(new DrawMode);
	spDrawMode->setDrawStyle(DrawMode::e_solid);
	spDrawMode->setLineWidth(3.0);
	spDrawMode->setPointSize(8.0);
//	spDrawMode->setAntialias(TRUE);

	m_spDrawOpenGL->pushDrawMode(spDrawMode);
}

void MetaBrush::draw(void)
{
	//* sandbox any OpenGL state changes
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	if(m_spDrawOpenGL.isNull())
	{
		m_spDrawOpenGL=m_pOperatorContext->master()->registry()->create(
				"DrawI.DrawOpenGL");

		setupDrawOpenGL();
	}

	sp<DrawMode> spDrawMode=m_spDrawOpenGL->popDrawMode();
	m_spDrawOpenGL->pushDrawMode(spDrawMode);

	//* perspective

	sp<ViewI> spViewI=m_spDrawOpenGL->view();
	spViewI->use(ViewMode::e_perspective);

	if(m_spDrawPerspective.isValid())
	{
		m_spDrawOpenGL->pushDrawMode(spDrawMode);
//		feLog("MetaBrush::draw perspective\n");
		m_spDrawPerspective->flushLive();
		m_spDrawOpenGL->popDrawMode();
	}

#if FALSE
//	feLog("axes 3D\n");
	m_spDrawOpenGL->drawAxes(1.0);

	SpatialTransform xform;
	setIdentity(xform);
	translate(xform,SpatialVector(0.2,0.3,0.4));
	rotate(xform,-90.0*degToRad,e_xAxis);

	const SpatialVector scale(0.1,0.1,0.1);
	const Color magenta(1.0,0.0,1.0,1.0);

//	feLog("sphere 3D\n");
	m_spDrawOpenGL->drawSphere(xform,&scale,magenta);

//	feLog("circle 3D\n");
	translate(xform,SpatialVector(0.2,0.3,0.0));
	m_spDrawOpenGL->drawCircle(xform,&scale,magenta);
#endif

	//* ortho
	sp<ViewMode> spViewMode=spViewI->viewMode();
	spViewMode->setViewport(m_viewport);
	spViewI->use(ViewMode::e_ortho);

	if(m_spDrawOrtho.isValid())
	{
		m_spDrawOpenGL->pushDrawMode(spDrawMode);
//		feLog("MetaBrush::draw ortho\n");
		m_spDrawOrtho->flushLive();
		m_spDrawOpenGL->popDrawMode();
	}

#if FALSE
	const Color yellow(1.0f,1.0f,0.0f,1.0f);
	const SpatialVector circleScale(10.0f,10.0f,10.0f);
	const SpatialVector center(130.0,160.0,1.0f);

//	feLog("circle 2D\n");
	SpatialTransform transform;
	setIdentity(transform);
	translate(transform,center);
	m_spDrawOpenGL->drawCircle(transform,&circleScale,yellow);

	const Color cyan(0.0f,1.0f,1.0f,1.0f);
	m_spDrawOpenGL->drawAlignedText(
			SpatialVector(100.0f,120.0f,1.0f),"Hello",cyan);

	const Color darkred(0.5f,0.0f,0.0f,1.0f);
	const SpatialVector dot(30.0,70.0,1.0);
	m_spDrawOpenGL->drawPoints(&dot,NULL,1,FALSE,&darkred);
#endif

	//* restore original state
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glPopAttrib();
}
